@@ -30,6 +30,9 @@
 #include <QStringList>
 #include <QUrl>
 #include <QMultiHash>
+#include <QLoggingCategory>
+
+Q_LOGGING_CATEGORY(APPSTREAMQT_DB, "appstreamqt.database")
 
 using namespace Appstream;
 
@@ -113,16 +116,17 @@ Component xapianDocToComponent(Xapian::Document document) {
         }
         component.setUrls(urls);
     } else {
-        qWarning("Bad url strings for component: '%s' (%s)", qPrintable(id), qPrintable(concatUrlStrings));
+        qCWarning(APPSTREAMQT_DB, "Bad url strings for component: '%s' (%s)", qPrintable(id), qPrintable(concatUrlStrings));
     }
 
+    // Provided items
     QString concatProvides = value(document, XapianValues::PROVIDED_ITEMS);
     QStringList providesList = concatProvides.split('\n',QString::SkipEmptyParts);
     QList<Provides> provideslist;
     Q_FOREACH(const QString& string, providesList) {
         QStringList providesParts = string.split(';',QString::SkipEmptyParts);
         if(providesParts.size() < 2) {
-            qWarning("Bad component parts for component: '%s' (%s)", qPrintable(id), qPrintable(string));
+            qCWarning(APPSTREAMQT_DB, "Bad component parts for component: '%s' (%s)", qPrintable(id), qPrintable(string));
             continue;
         }
         QString kindString = providesParts.takeFirst();
@@ -137,6 +141,21 @@ Component xapianDocToComponent(Xapian::Document document) {
     }
     component.setProvides(provideslist);
 
+    // Bundles
+    QString concatBundleIds = value(document, XapianValues::BUNDLES);
+    QStringList bundleIds = concatBundleIds.split('\n',QString::SkipEmptyParts);
+    if(bundleIds.size() %2 == 0) {
+        QHash<Component::BundleKind, QString> bundles;
+        for(int i = 0; i < bundleIds.size(); i=i+2) {
+            Component::BundleKind bkind = Component::stringToBundleKind(bundleIds.at(i));
+            QString bdid = QString(bundleIds.at(i+1));
+            bundles.insertMulti(bkind, bdid);
+        }
+        component.setBundles(bundles);
+    } else {
+        qCWarning(APPSTREAMQT_DB, "Bad bundle strings for component: '%s' (%s)", qPrintable(id), qPrintable(concatBundleIds));
+    }
+
     // Stock icon
     QString icon = value(document,XapianValues::ICON);
     component.setIcon(icon);
@@ -152,7 +171,7 @@ Component xapianDocToComponent(Xapian::Document document) {
             component.addIconUrl(url, size);
         }
     } else {
-        qWarning("Bad icon-url strings for component: '%s' (%s)", qPrintable(id), qPrintable(concatIconUrlStrings));
+        qCWarning(APPSTREAMQT_DB, "Bad icon-url strings for component: '%s' (%s)", qPrintable(id), qPrintable(concatIconUrlStrings));
     }
 
     // Summary
@@ -185,6 +204,10 @@ Component xapianDocToComponent(Xapian::Document document) {
     QString projectGroup = value(document,XapianValues::PROJECT_GROUP);
     component.setProjectGroup(projectGroup);
 
+    // Developer name
+    QString developerName = value(document,XapianValues::DEVELOPER_NAME);
+    component.setDeveloperName(developerName);
+
     // Releases data
     QString releasesXml = value(document,XapianValues::RELEASES_DATA);
     Q_UNUSED(releasesXml);
@@ -226,7 +249,7 @@ Component Database::componentById(const QString& id) const {
 
     Xapian::MSet matches = enquire.get_mset (0, d->m_db.get_doccount ());
     if (matches.size () > 1) {
-        qWarning ("Found more than one component with id '%s'! Returning the first one.", qPrintable(id));
+        qCWarning(APPSTREAMQT_DB, "Found more than one component with id '%s'! Returning the first one.", qPrintable(id));
         Q_ASSERT(false);
     }
     if (matches.empty()) {
@@ -377,9 +400,6 @@ QList<Component> Database::findComponentsByPackageName(const QString& packageNam
 }
 
 
-Database::Database() : d(new DatabasePrivate(QLatin1String("/var/cache/app-info/xapian"))) {
+Database::Database() : d(new DatabasePrivate(QLatin1String("/var/cache/app-info/xapian/C"))) {
 
 }
-
-
-#include "database.moc"
