@@ -1,6 +1,6 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2014 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2015 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -39,37 +39,32 @@
 #include "as-distro-details.h"
 #include "as-settings-private.h"
 
-#include "data-providers/appstream-xml.h"
+#include "as-metadata.h"
+#include "as-metadata-private.h"
 #ifdef DEBIAN_DEP11
-#include "data-providers/debian-dep11.h"
-#endif
-#ifdef UBUNTU_APPINSTALL
-#include "data-providers/ubuntu-appinstall.h"
+#include "as-dep11.h"
 #endif
 
-const gchar* AS_APPSTREAM_XML_PATHS[4] = {AS_APPSTREAM_BASE_PATH "/xmls",
-										"/var/cache/app-info/xmls",
-										"/var/lib/app-info/xmls",
-										NULL};
-const gchar* AS_APPSTREAM_DEP11_PATHS[4] = {AS_APPSTREAM_BASE_PATH "/yaml",
-										"/var/cache/app-info/yaml",
-										"/var/lib/app-info/yaml",
-										NULL};
-
-#define AS_PROVIDER_UBUNTU_APPINSTALL_DIR "/usr/share/app-install"
+const gchar *AS_APPSTREAM_XML_PATHS[4] = {AS_APPSTREAM_BASE_PATH "/xmls",
+						"/var/cache/app-info/xmls",
+						"/var/lib/app-info/xmls",
+						NULL};
+const gchar *AS_APPSTREAM_DEP11_PATHS[4] = {AS_APPSTREAM_BASE_PATH "/yaml",
+						"/var/cache/app-info/yaml",
+						"/var/lib/app-info/yaml",
+						NULL};
 
 typedef struct _AsDataPoolPrivate	AsDataPoolPrivate;
+
 struct _AsDataPoolPrivate
 {
 	GHashTable* cpt_table;
 	GPtrArray* providers;
 	gchar *scr_base_url;
-	gboolean initialized;
 	gchar *locale;
 
 	gchar **asxml_paths;
 	gchar **dep11_paths;
-	gchar **appinstall_paths;
 
 	gchar **icon_paths;
 };
@@ -91,8 +86,8 @@ as_data_pool_finalize (GObject *object)
 	g_free (priv->scr_base_url);
 	g_hash_table_unref (priv->cpt_table);
 
-	g_strfreev (priv->appinstall_paths);
 	g_strfreev (priv->asxml_paths);
+	g_strfreev (priv->dep11_paths);
 
 	g_strfreev (priv->icon_paths);
 
@@ -117,8 +112,13 @@ as_data_pool_class_init (AsDataPoolClass *klass)
 	object_class->finalize = as_data_pool_finalize;
 }
 
+/**
+ * as_data_pool_add_new_component:
+ *
+ * Register a new component in the global AppStream metadata pool.
+ */
 static void
-as_data_pool_new_component_cb (AsDataProvider *sender, AsComponent* cpt, AsDataPool *dpool)
+as_data_pool_add_new_component (AsDataPool *dpool, AsComponent *cpt)
 {
 	const gchar *cpt_id;
 	AsComponent *existing_cpt;
@@ -137,59 +137,25 @@ as_data_pool_new_component_cb (AsDataProvider *sender, AsComponent* cpt, AsDataP
 		priority = as_component_get_priority (existing_cpt);
 		if (priority < as_component_get_priority (cpt)) {
 			g_hash_table_replace (priv->cpt_table,
-								  g_strdup (cpt_id),
-								  g_object_ref (cpt));
+						g_strdup (cpt_id),
+						g_object_ref (cpt));
 		} else {
-			g_debug ("Detected colliding ids: %s was already added.", cpt_id);
+			if ((!as_component_has_bundle (existing_cpt)) && (as_component_has_bundle (cpt))) {
+				GHashTable *bundles;
+				/* propagate bundle information to existing component */
+				bundles = as_component_get_bundle_ids (cpt);
+				as_component_set_bundles_table (existing_cpt, bundles);
+			} else {
+				g_debug ("Detected colliding ids: %s was already added.", cpt_id);
+			}
 		}
 	} else {
 		g_hash_table_insert (priv->cpt_table,
-							g_strdup (cpt_id),
-							g_object_ref (cpt));
+					g_strdup (cpt_id),
+					g_object_ref (cpt));
 	}
 }
 
-/**
- * as_data_pool_initialize:
- *
- * Initialize the pool with the predefined metadata locations.
- **/
-void
-as_data_pool_initialize (AsDataPool *dpool)
-{
-	AsDataProvider *dprov;
-	guint i;
-	AsDataPoolPrivate *priv = GET_PRIVATE (dpool);
-
-	/* regenerate data providers, in case someone is calling init twice */
-	g_ptr_array_unref (priv->providers);
-	priv->providers = g_ptr_array_new_with_free_func (g_object_unref);
-
-	/* added by priority: Appstream XML has the highest, Ubuntu AppInstall the lowest priority */
-	dprov = AS_DATA_PROVIDER (as_provider_xml_new ());
-	as_data_provider_set_watch_files (dprov, priv->asxml_paths);
-	g_ptr_array_add (priv->providers, dprov);
-#ifdef DEBIAN_DEP11
-	dprov = AS_DATA_PROVIDER (as_provider_dep11_new ());
-	as_data_provider_set_watch_files (dprov, priv->dep11_paths);
-	g_ptr_array_add (priv->providers, dprov);
-#endif
-#ifdef UBUNTU_APPINSTALL
-	dprov = AS_DATA_PROVIDER (as_provider_ubuntu_appinstall_new ());
-	as_data_provider_set_watch_files (dprov, priv->appinstall_paths);
-	g_ptr_array_add (priv->providers, dprov);
-#endif
-	dprov = NULL;
-
-	/* connect all data provider signals */
-	for (i = 0; i < priv->providers->len; i++) {
-		dprov = (AsDataProvider*) g_ptr_array_index (priv->providers, i);
-		g_signal_connect_object (dprov, "component", (GCallback) as_data_pool_new_component_cb, dpool, 0);
-	}
-
-	priv->initialized = TRUE;
-}
-
 /**
  * as_data_pool_get_watched_locations:
  * @dpool: a valid #AsDataPool instance
@@ -201,8 +167,6 @@ as_data_pool_initialize (AsDataPool *dpool)
 gchar**
 as_data_pool_get_watched_locations (AsDataPool *dpool)
 {
-	AsDataProvider *dprov;
-	gchar **wfiles;
 	guint i;
 	GPtrArray *res_array;
 	gchar **res;
@@ -210,23 +174,235 @@ as_data_pool_get_watched_locations (AsDataPool *dpool)
 	g_return_val_if_fail (dpool != NULL, NULL);
 
 	res_array = g_ptr_array_new_with_free_func (g_free);
-	for (i = 0; i < priv->providers->len; i++) {
-		guint j;
-		dprov = (AsDataProvider*) g_ptr_array_index (priv->providers, i);
-		wfiles = as_data_provider_get_watch_files (dprov);
-		/* if there is nothing to watch for, we can just continue here */
-		if (wfiles == NULL)
-			continue;
-		for (j = 0; wfiles[j] != NULL; j++) {
-			g_ptr_array_add (res_array, g_strdup (wfiles[j]));
-		}
+	for (i = 0; priv->asxml_paths[i] != NULL; i++) {
+		g_ptr_array_add (res_array, g_strdup (priv->asxml_paths[i]));
 	}
+#ifdef DEBIAN_DEP11
+	for (i = 0; priv->dep11_paths[i] != NULL; i++) {
+		g_ptr_array_add (res_array, g_strdup (priv->dep11_paths[i]));
+	}
+#endif
 
 	res = as_ptr_array_to_strv (res_array);
 	g_ptr_array_unref (res_array);
 	return res;
 }
 
+/**
+ * as_data_pool_read_asxml:
+ */
+static gboolean
+as_data_pool_read_asxml (AsDataPool *dpool)
+{
+	GPtrArray *xml_files;
+	GPtrArray *components;
+	guint i;
+	GFile *infile;
+	gboolean ret = TRUE;
+	const gchar *content_type;
+	AsMetadata *metad;
+	GError *error = NULL;
+	AsDataPoolPrivate *priv = GET_PRIVATE (dpool);
+
+	xml_files = g_ptr_array_new_with_free_func (g_free);
+
+	if ((priv->asxml_paths == NULL) || (priv->asxml_paths[0] == NULL))
+		return TRUE;
+
+	for (i = 0; priv->asxml_paths[i] != NULL; i++) {
+		gchar *path;
+		GPtrArray *xmls;
+		guint j;
+		path = priv->asxml_paths[i];
+
+		if (!g_file_test (path, G_FILE_TEST_EXISTS))
+			continue;
+
+		xmls = as_utils_find_files_matching (path, "*.xml*", FALSE);
+		if (xmls == NULL)
+			continue;
+		for (j = 0; j < xmls->len; j++) {
+			const gchar *val;
+			val = (const gchar *) g_ptr_array_index (xmls, j);
+			g_ptr_array_add (xml_files, g_strdup (val));
+		}
+
+		g_ptr_array_unref (xmls);
+	}
+
+	/* check if we have XML data */
+	if (xml_files->len == 0) {
+		g_ptr_array_unref (xml_files);
+		return TRUE;
+	}
+
+	metad = as_metadata_new ();
+	as_metadata_set_parser_mode (metad, AS_PARSER_MODE_DISTRO);
+	as_metadata_set_locale (metad, priv->locale);
+
+	for (i = 0; i < xml_files->len; i++) {
+		gchar *fname;
+		GFileInfo *info = NULL;
+
+		fname = (gchar*) g_ptr_array_index (xml_files, i);
+		infile = g_file_new_for_path (fname);
+		if (!g_file_query_exists (infile, NULL)) {
+			g_warning ("File '%s' does not exist.", fname);
+			g_object_unref (infile);
+			continue;
+		}
+
+		info = g_file_query_info (infile,
+				G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+				G_FILE_QUERY_INFO_NONE,
+				NULL, NULL);
+		if (info == NULL) {
+			g_debug ("No info for file '%s' found, file was skipped.", fname);
+			g_object_unref (infile);
+			continue;
+		}
+		content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+		if ((g_strcmp0 (content_type, "application/xml") == 0) || (g_strcmp0 (content_type, "text/plain") == 0) ||
+			(g_strcmp0 (content_type, "application/gzip") == 0 || g_strcmp0 (content_type, "application/x-gzip") == 0)) {
+			as_metadata_parse_file (metad, infile, &error);
+			if (error != NULL) {
+				g_debug ("ASXML-WARNING: %s", error->message);
+				g_error_free (error);
+				error = NULL;
+				ret = FALSE;
+			}
+		} else {
+			g_warning ("Invalid file of type '%s' found. File '%s' was skipped.", content_type, fname);
+		}
+		g_object_unref (info);
+		g_object_unref (infile);
+
+		if (!ret)
+			break;
+	}
+
+	components = as_metadata_get_components (metad);
+	for (i = 0; i < components->len; i++) {
+		as_data_pool_add_new_component (dpool,
+						AS_COMPONENT (g_ptr_array_index (components, i)));
+	}
+
+	g_ptr_array_unref (xml_files);
+	g_object_unref (metad);
+
+	return ret;
+}
+
+#ifdef DEBIAN_DEP11
+/**
+ * as_data_pool_read_dep11:
+ */
+static gboolean
+as_data_pool_read_dep11 (AsDataPool *dpool)
+{
+	AsDEP11 *dep11;
+	GPtrArray *yaml_files;
+	GPtrArray *components;
+	guint i;
+	GFile *infile;
+	gboolean ret = TRUE;
+	const gchar *content_type;
+	GError *error = NULL;
+	AsDataPoolPrivate *priv = GET_PRIVATE (dpool);
+
+	yaml_files = g_ptr_array_new_with_free_func (g_free);
+
+	if ((priv->dep11_paths == NULL) || (priv->dep11_paths[0] == NULL))
+		return TRUE;
+
+	for (i = 0; priv->dep11_paths[i] != NULL; i++) {
+		gchar *path;
+		GPtrArray *yamls;
+		guint j;
+		path = priv->dep11_paths[i];
+
+		if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
+			continue;
+		}
+
+		yamls = as_utils_find_files_matching (path, "*.yml*", FALSE);
+		if (yamls == NULL)
+			continue;
+		for (j = 0; j < yamls->len; j++) {
+			const gchar *val;
+			val = (const gchar *) g_ptr_array_index (yamls, j);
+			g_ptr_array_add (yaml_files, g_strdup (val));
+		}
+
+		g_ptr_array_unref (yamls);
+	}
+
+	/* do we have metadata at all? */
+	if (yaml_files->len == 0) {
+		g_ptr_array_unref (yaml_files);
+		return TRUE;
+	}
+
+	dep11 = as_dep11_new ();
+
+	ret = TRUE;
+	for (i = 0; i < yaml_files->len; i++) {
+		gchar *fname;
+		GFileInfo *info = NULL;
+
+		fname = (gchar*) g_ptr_array_index (yaml_files, i);
+		infile = g_file_new_for_path (fname);
+		if (!g_file_query_exists (infile, NULL)) {
+			g_warning ("File '%s' does not exist.", fname);
+			g_object_unref (infile);
+			continue;
+		}
+
+		info = g_file_query_info (infile,
+				G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+				G_FILE_QUERY_INFO_NONE,
+				NULL, NULL);
+		if (info == NULL) {
+			g_debug ("No info for file '%s' found, file was skipped.", fname);
+			g_object_unref (infile);
+			continue;
+		}
+		content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
+		if ((g_strcmp0 (content_type, "application/x-yaml") == 0) || (g_strcmp0 (content_type, "text/plain") == 0) ||
+			(g_strcmp0 (content_type, "application/gzip") == 0) || (g_strcmp0 (content_type, "application/x-gzip") == 0)) {
+			as_dep11_parse_file (dep11, infile, &error);
+			if (error != NULL) {
+				g_debug ("DEP11-WARNING: %s", error->message);
+				g_error_free (error);
+				error = NULL;
+				ret = FALSE;
+			}
+		} else {
+			g_warning ("Invalid file of type '%s' found. File '%s' was skipped.", content_type, fname);
+		}
+		g_object_unref (info);
+		g_object_unref (infile);
+
+		if (!ret)
+			break;
+	}
+
+	components = as_dep11_get_components (dep11);
+	for (i = 0; i < components->len; i++) {
+		as_data_pool_add_new_component (dpool,
+						AS_COMPONENT (g_ptr_array_index (components, i)));
+	}
+
+	g_object_unref (dep11);
+	g_ptr_array_unref (yaml_files);
+
+	return ret;
+}
+#endif
+
+
+
+
 /**
  * as_data_pool_update:
  *
@@ -239,16 +415,9 @@ as_data_pool_get_watched_locations (AsDataPool *dpool)
 gboolean
 as_data_pool_update (AsDataPool *dpool)
 {
-	guint i;
-	AsDataProvider *dprov;
 	gboolean ret = TRUE;
 	AsDataPoolPrivate *priv = GET_PRIVATE (dpool);
 
-	if (!priv->initialized) {
-		g_error ("DataPool has never been initialized and can not find metadata.");
-		return FALSE;
-	}
-
 	/* just in case, clear the components table */
 	g_hash_table_unref (priv->cpt_table);
 	priv->cpt_table = g_hash_table_new_full (g_str_hash,
@@ -256,16 +425,12 @@ as_data_pool_update (AsDataPool *dpool)
 						g_free,
 						(GDestroyNotify) g_object_unref);
 
-	/* call all AppStream data providers to return components they find */
-	for (i = 0; i < priv->providers->len; i++) {
-		gboolean dret;
-		dprov = (AsDataProvider*) g_ptr_array_index (priv->providers, i);
-		as_data_provider_set_locale (dprov, priv->locale);
-
-		dret = as_data_provider_execute (dprov);
-		if (!dret)
-			ret = FALSE;
-	}
+	/* read all AppStream metadata that we can find */
+	ret = as_data_pool_read_asxml (dpool);
+#ifdef DEBIAN_DEP11
+	if (!as_data_pool_read_dep11 (dpool))
+		ret = FALSE;
+#endif
 
 	return ret;
 }
@@ -284,6 +449,23 @@ as_data_pool_get_components (AsDataPool *dpool)
 	return g_hash_table_get_values (priv->cpt_table);
 }
 
+/**
+ * as_data_pool_get_component_by_id:
+ *
+ * Get a specific component by its ID.
+ *
+ * Returns: (transfer full): An #AsComponent
+ */
+AsComponent*
+as_data_pool_get_component_by_id (AsDataPool *dpool, const gchar *id)
+{
+	AsDataPoolPrivate *priv = GET_PRIVATE (dpool);
+	if (id == NULL)
+		return NULL;
+
+	return g_object_ref (AS_COMPONENT (g_hash_table_lookup (priv->cpt_table, id)));
+}
+
 /**
  * as_data_pool_set_locale:
  * @dpool: a #AsDataPool instance.
@@ -370,10 +552,6 @@ as_data_pool_set_data_source_directories (AsDataPool *dpool, gchar **dirs)
 	g_strfreev (priv->icon_paths);
 	priv->icon_paths = as_ptr_array_to_strv (icondirs);
 
-	/* nuke AppInstall search, in case the provider is enabled */
-	g_strfreev (priv->appinstall_paths);
-	priv->appinstall_paths = NULL;
-
 	g_ptr_array_unref (xmldirs);
 	g_ptr_array_unref (yamldirs);
 }
@@ -402,9 +580,9 @@ as_data_pool_new (void)
 	priv->locale = as_get_locale ();
 
 	priv->cpt_table = g_hash_table_new_full (g_str_hash,
-								g_str_equal,
-								g_free,
-								(GDestroyNotify) g_object_unref);
+						g_str_equal,
+						g_free,
+						(GDestroyNotify) g_object_unref);
 	priv->providers = g_ptr_array_new_with_free_func (g_object_unref);
 
 	distro = as_distro_details_new ();
@@ -435,14 +613,8 @@ as_data_pool_new (void)
 			priv->dep11_paths[i] = NULL;
 	}
 
-	/* set default directories for Ubuntu AppInstall */
-	priv->appinstall_paths = g_new0 (gchar*, 2);
-	priv->appinstall_paths[0] = g_strdup (AS_PROVIDER_UBUNTU_APPINSTALL_DIR);
-
 	/* set default icon search locations */
 	priv->icon_paths = as_distro_details_get_icon_repository_paths ();
 
-	priv->initialized = FALSE;
-
 	return AS_DATA_POOL (dpool);
 }
