@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2012-2014 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -73,3 +73,41 @@ as_url_kind_from_string (const gchar *url_kind)
 		return AS_URL_KIND_DONATION;
 	return AS_URL_KIND_UNKNOWN;
 }
+
+/**
+ * as_bundle_kind_to_string:
+ * @bundle_kind: the %AsBundleKind.
+ *
+ * Converts the enumerated value to an text representation.
+ *
+ * Returns: string version of @bundle_kind
+ *
+ * Since: 0.8.0
+ **/
+const gchar *
+as_bundle_kind_to_string (AsBundleKind bundle_kind)
+{
+	if (bundle_kind == AS_BUNDLE_KIND_LIMBA)
+		return "limba";
+	if (bundle_kind == AS_BUNDLE_KIND_XDG_APP)
+		return "xdg-app";
+	return "unknown";
+}
+
+/**
+ * as_bundle_kind_from_string:
+ * @bundle_kind: the string.
+ *
+ * Converts the text representation to an enumerated value.
+ *
+ * Returns: a #AsBundleKind or %AS_BUNDLE_KIND_UNKNOWN for unknown
+ **/
+AsBundleKind
+as_bundle_kind_from_string (const gchar *bundle_kind)
+{
+	if (g_strcmp0 (bundle_kind, "limba") == 0)
+		return AS_BUNDLE_KIND_LIMBA;
+	if (g_strcmp0 (bundle_kind, "xdg-app") == 0)
+		return AS_BUNDLE_KIND_LIMBA;
+	return AS_BUNDLE_KIND_UNKNOWN;
+}
