@@ -1,6 +1,6 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2014 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2015 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -31,8 +31,8 @@
 
 #include "as-component.h"
 
-#define AS_TYPE_METADATA			(as_metadata_get_type())
-#define AS_METADATA(obj)			(G_TYPE_CHECK_INSTANCE_CAST((obj), AS_TYPE_METADATA, AsMetadata))
+#define AS_TYPE_METADATA		(as_metadata_get_type())
+#define AS_METADATA(obj)		(G_TYPE_CHECK_INSTANCE_CAST((obj), AS_TYPE_METADATA, AsMetadata))
 #define AS_METADATA_CLASS(cls)		(G_TYPE_CHECK_CLASS_CAST((cls), AS_TYPE_METADATA, AsMetadataClass))
 #define AS_IS_METADATA(obj)		(G_TYPE_CHECK_INSTANCE_TYPE((obj), AS_TYPE_METADATA))
 #define AS_IS_METADATA_CLASS(cls)	(G_TYPE_CHECK_CLASS_TYPE((cls), AS_TYPE_METADATA))
@@ -73,21 +73,43 @@ typedef enum {
 	AS_METADATA_ERROR_LAST
 } AsMetadataError;
 
-#define	AS_METADATA_ERROR				as_metadata_error_quark ()
+#define	AS_METADATA_ERROR		as_metadata_error_quark ()
 
-GType		 	as_metadata_get_type	(void);
-AsMetadata*		as_metadata_new			(void);
+GType		 	as_metadata_get_type (void);
+AsMetadata*		as_metadata_new (void);
 GQuark			as_metadata_error_quark	(void);
 
-AsComponent*	as_metadata_parse_file (AsMetadata* metad,
-										GFile* infile,
-										GError **error);
-AsComponent*	as_metadata_parse_data (AsMetadata* metad,
-										const gchar *data,
-										GError **error);
+void			as_metadata_parse_file (AsMetadata *metad,
+							GFile *file,
+							GError **error);
+void			as_metadata_parse_data (AsMetadata *metad,
+							const gchar *data,
+							GError **error);
+
+AsComponent		*as_metadata_get_component (AsMetadata *metad);
+GPtrArray		*as_metadata_get_components (AsMetadata *metad);
+
+void			as_metadata_clear_components (AsMetadata *metad);
+
+void			as_metadata_add_component (AsMetadata *metad,
+							AsComponent *cpt);
+gchar			*as_metadata_component_to_upstream_xml (AsMetadata *metad);
+gchar			*as_metadata_components_to_distro_xml (AsMetadata *metad);
+
+void			as_metadata_save_upstream_xml (AsMetadata *metad,
+							const gchar *fname,
+							GError **error);
+void			as_metadata_save_distro_xml (AsMetadata *metad,
+							const gchar *fname,
+							GError **error);
+
 void			as_metadata_set_locale (AsMetadata *metad,
-										const gchar *locale);
-const gchar*	as_metadata_get_locale (AsMetadata *metad);
+							const gchar *locale);
+const gchar		*as_metadata_get_locale (AsMetadata *metad);
+
+const gchar		*as_metadata_get_origin (AsMetadata *metad);
+void			as_metadata_set_origin (AsMetadata *metad,
+							const gchar *origin);
 
 
 G_END_DECLS
