@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2012-2014 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -27,11 +27,10 @@
 G_BEGIN_DECLS
 
 #define AS_DB_SCHEMA_VERSION "1"
-#define AS_APPSTREAM_BASE_PATH DATADIR "/app-info"
 #define AS_CONFIG_NAME "/etc/appstream.conf"
 #define AS_APPSTREAM_CACHE_PATH "/var/cache/app-info"
 
-#define AS_APPSTREAM_BASE_PATH DATADIR "/app-info"
+#define AS_APPSTREAM_BASE_PATH "/usr/share/app-info"
 
 G_END_DECLS
 
