@@ -1,6 +1,6 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2012-2014 Matthias Klumpp <matthias@tenstral.net>
+ * Copyright (C) 2012-2015 Matthias Klumpp <matthias@tenstral.net>
  *
  * Licensed under the GNU Lesser General Public License Version 2.1
  *
@@ -25,6 +25,14 @@
 
 G_BEGIN_DECLS
 
+#define GS_DEFINE_CLEANUP_FUNCTION(Type, name, func) \
+  static inline void name (void *v) \
+  { \
+    func (*(Type*)v); \
+  }
+GS_DEFINE_CLEANUP_FUNCTION(void*, gs_local_free, g_free)
+#define _cleanup_free_ __attribute__ ((cleanup(gs_local_free)))
+
 gboolean		as_utils_touch_dir (const gchar* dirname);
 gboolean		as_utils_delete_dir_recursive (const gchar* dirname);
 gchar*			as_string_strip (const gchar* str);
