@@ -27,32 +27,36 @@ namespace Appstream {
 namespace XapianValues {
 
 enum XapianValues {
-	TYPE = 140,
-	IDENTIFIER = 141,
-	CPTNAME = 142,
-	CPTNAME_UNTRANSLATED = 143,
-	PKGNAME = 144,
+	TYPE = 120,
+	IDENTIFIER = 121,
+	CPTNAME = 122,
+	CPTNAME_UNTRANSLATED = 123,
+	PKGNAME = 124,
+	SOURCE_PKGNAME = 125,
+	BUNDLES = 126,
 
-	SUMMARY = 145,
-	DESCRIPTION = 146,
-	CATEGORIES = 147,
+	SUMMARY = 127,
+	DESCRIPTION = 128,
+	CATEGORIES = 129,
 
-	ICON = 148,
-	ICON_URLS = 149,
+	ICON = 130,
+	ICON_URLS = 131,
 
-	PROVIDED_ITEMS = 150,
-	SCREENSHOT_DATA = 152, // screenshot definitions, as XML
-	RELEASES_DATA = 153, // releases definitions, as XML
+	PROVIDED_ITEMS = 140,
+	SCREENSHOT_DATA = 141, // screenshot definitions, as XML
+	RELEASES_DATA = 142, // releases definitions, as XML
 
-	LICENSE = 154,
-	URLS = 155,
+	LICENSE = 150,
+	URLS = 151,
 
 	PROJECT_GROUP = 160,
+	DEVELOPER_NAME = 161,
 
 	COMPULSORY_FOR = 170,
 	LANGUAGES = 171,
 
-	ORIGIN = 180
+	ORIGIN = 180,
+
 
 //!	GETTEXT_DOMAIN = 180,
 //!	ARCHIVE_SECTION = 181,
