@@ -59,13 +59,25 @@ DatabaseRead::open (const gchar *dbPath)
 		return false;
 	}
 
+	m_dbLocale = m_xapianDB.get_metadata ("db-locale");
+	if (m_dbLocale.empty ())
+		m_dbLocale = "C";
+
+	m_schemaVersion = m_xapianDB.get_metadata ("db-schema-version");
+
 	return true;
 }
 
 string
 DatabaseRead::getSchemaVersion ()
 {
-	return m_xapianDB.get_metadata ("db-schema-version");
+	return m_schemaVersion;
+}
+
+string
+DatabaseRead::getLocale ()
+{
+	return m_dbLocale;
 }
 
 AsComponent*
@@ -74,6 +86,9 @@ DatabaseRead::docToComponent (Xapian::Document doc)
 	AsComponent *cpt = as_component_new ();
 	string str;
 
+	/* set component active languge (which is the locale the database was built for) */
+	as_component_set_active_locale (cpt, m_dbLocale.c_str ());
+
 	// Component type/kind
 	string type_str = doc.get_value (XapianValues::TYPE);
 	as_component_set_kind (cpt, as_component_kind_from_string (type_str.c_str ()));
@@ -84,7 +99,9 @@ DatabaseRead::docToComponent (Xapian::Document doc)
 
 	// Component name
 	string cptName = doc.get_value (XapianValues::CPTNAME);
-	as_component_set_name (cpt, cptName.c_str ());
+	as_component_set_name (cpt, cptName.c_str (), NULL);
+	cptName = doc.get_value (XapianValues::CPTNAME_UNTRANSLATED);
+	as_component_set_name (cpt, cptName.c_str (), "C");
 
 	// Package name
 	string pkgNamesStr = doc.get_value (XapianValues::PKGNAME);
@@ -92,13 +109,13 @@ DatabaseRead::docToComponent (Xapian::Document doc)
 	as_component_set_pkgnames (cpt, pkgs);
 	g_strfreev (pkgs);
 
-	// Untranslated application name
-	string appname_orig = doc.get_value (XapianValues::CPTNAME_UNTRANSLATED);
-	as_component_set_name_original (cpt, appname_orig.c_str ());
+	// Source package name
+	string cptSPkg = doc.get_value (XapianValues::SOURCE_PKGNAME);
+	as_component_set_source_pkgname (cpt, cptSPkg.c_str ());
 
 	// Origin
 	string cptOrigin = doc.get_value (XapianValues::ORIGIN);
-	as_component_set_name (cpt, cptOrigin.c_str ());
+	as_component_set_origin (cpt, cptOrigin.c_str ());
 
 	// URLs
 	str = doc.get_value (XapianValues::URLS);
@@ -113,9 +130,22 @@ DatabaseRead::docToComponent (Xapian::Document doc)
 	}
 	g_strfreev (urls);
 
+	// Bundles
+	str = doc.get_value (XapianValues::BUNDLES);
+	gchar **bundle_ids = g_strsplit (str.c_str (), "\n", -1);
+	for (uint i = 0; bundle_ids[i] != NULL; i += 2) {
+		/* bundle-ids are stored in form of "type \n id" (so we just need one stringsplit here...) */
+		if (bundle_ids[i+1] == NULL)
+			break;
+		AsBundleKind bkind = as_bundle_kind_from_string (bundle_ids[i]);
+		if (bkind != AS_BUNDLE_KIND_UNKNOWN)
+			as_component_add_bundle_id (cpt, bkind, bundle_ids[i+1]);
+	}
+	g_strfreev (bundle_ids);
+
 	// Stock icon
 	string appIcon = doc.get_value (XapianValues::ICON);
-	as_component_set_icon (cpt, appIcon.c_str ());
+	as_component_add_icon (cpt, AS_ICON_KIND_STOCK, 0, 0, appIcon.c_str ());
 
 	// Icon urls
 	str = doc.get_value (XapianValues::ICON_URLS);
@@ -133,11 +163,11 @@ DatabaseRead::docToComponent (Xapian::Document doc)
 
 	// Summary
 	string appSummary = doc.get_value (XapianValues::SUMMARY);
-	as_component_set_summary (cpt, appSummary.c_str ());
+	as_component_set_summary (cpt, appSummary.c_str (), NULL);
 
 	// Long description
 	string appDescription = doc.get_value (XapianValues::DESCRIPTION);
-	as_component_set_description (cpt, appDescription.c_str ());
+	as_component_set_description (cpt, appDescription.c_str (), NULL);
 
 	// Categories
 	string categories_str = doc.get_value (XapianValues::CATEGORIES);
@@ -173,6 +203,10 @@ DatabaseRead::docToComponent (Xapian::Document doc)
 	string project_group = doc.get_value (XapianValues::PROJECT_GROUP);
 	as_component_set_project_group (cpt, project_group.c_str ());
 
+	// Source package name
+	string developerName = doc.get_value (XapianValues::DEVELOPER_NAME);
+	as_component_set_developer_name (cpt, developerName.c_str (), NULL);
+
 	// Releases data
 	string releases_xml = doc.get_value (XapianValues::RELEASES_DATA);
 	as_component_load_releases_from_internal_xml (cpt, releases_xml.c_str ());
