@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2012-2014 Matthias Klumpp <matthias@tenstral.net>
  *
@@ -67,24 +67,33 @@ void
 test_component ()
 {
 	AsComponent *cpt;
+	AsMetadata *metad;
 	gchar *str;
+	gchar *str2;
 	gchar **strv;
 
 	cpt = as_component_new ();
 	as_component_set_kind (cpt, AS_COMPONENT_KIND_DESKTOP_APP);
 
-	as_component_set_name (cpt, "Test");
-	as_component_set_summary (cpt, "It does things");
+	as_component_set_name (cpt, "Test", NULL);
+	as_component_set_summary (cpt, "It does things", NULL);
 
 	strv = _get_dummy_strv ("fedex");
 	as_component_set_pkgnames (cpt, strv);
 	g_strfreev (strv);
 
-	str = as_component_to_xml (cpt);
-	g_debug ("%s", str);
+	metad = as_metadata_new ();
+	as_metadata_add_component (metad, cpt);
+	str = as_metadata_component_to_upstream_xml (metad);
+	str2 = as_metadata_components_to_distro_xml (metad);
+	g_object_unref (metad);
+	g_debug ("%s", str2);
+
+	g_assert_cmpstr (str, ==, "<?xml version=\"1.0\"?>\n<component type=\"desktop\"><name>Test</name><summary>It does things</summary><pkgname>fedex</pkgname></component>\n");
+	g_assert_cmpstr (str2, ==, "<?xml version=\"1.0\"?>\n<components version=\"0.8\"><component type=\"desktop\"><name>Test</name><summary>It does things</summary><pkgname>fedex</pkgname></component></components>\n");
 
-	g_assert (g_strcmp0 (str, "<?xml version=\"1.0\"?>\n<component type=\"desktop\"><name>Test</name><summary>It does things</summary><pkgname>fedex</pkgname></component>\n") == 0);
 	g_free (str);
+	g_free (str2);
 }
 
 int
@@ -98,7 +107,7 @@ main (int argc, char **argv)
 	}
 
 	g_assert (argv[1] != NULL);
-	datadir = g_build_filename (argv[1], "data", NULL);
+	datadir = g_build_filename (argv[1], "samples", NULL);
 	g_assert (g_file_test (datadir, G_FILE_TEST_EXISTS) != FALSE);
 
 	g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
